﻿using System;
using System.Collections.Generic;
using Autofac;

namespace LightweightAdapterDemo
{
    public interface ICommand
    {
        void Execute();
    }

    public class SaveCommand : ICommand
    {
        public void Execute()
        {
            Console.WriteLine("Saving the image...");
        }
    }

    public class OpenCommand : ICommand
    {
        public void Execute()
        {
            Console.WriteLine("Opening an image...");
        }
    }

    public class ToolbarButton
    {
        readonly ICommand _command;

        public ToolbarButton(ICommand command)
        {
            _command = command;
        }

        public void Click()
        {
            _command.Execute();
        }
    }

    public class EditorWindow
    {
        readonly IEnumerable<ToolbarButton> _toolbarButtons;

        public EditorWindow(IEnumerable<ToolbarButton> toolbarButtons)
        {
            _toolbarButtons = toolbarButtons;
        }

        public void Show()
        {
            foreach (var tb in _toolbarButtons)
                tb.Click();
        }
    }

    class Program
    {
        static void Main()
        {
            var builder = new ContainerBuilder();
            builder.RegisterType<SaveCommand>().As<ICommand>();
            builder.RegisterType<OpenCommand>().As<ICommand>();
            builder.RegisterAdapter<ICommand, ToolbarButton>(cmd => new ToolbarButton(cmd));
            builder.RegisterType<EditorWindow>();
            using (var container = builder.Build())
            {
                var window = container.Resolve<EditorWindow>();
                window.Show();
            }

            Console.WriteLine("Done. Press any key.");
            Console.ReadKey(true);
        }
    }
}
